
(*s Automata. *)

module type OrderedType = sig
  type t
  val compare : t -> t -> int
end

(*s Non-deterministic automata. *)

module type ND = sig
  type t
  type state
  type input

  val empty : state -> t
  val add_transition : t -> state -> input -> state -> t
  val add_epsilon_transition : t -> state -> state -> t

  val fold_states : ('a -> state -> 'a) -> 'a -> t -> 'a
  val fold_transitions : 
    ('a -> state -> input -> state -> 'a) ->
      ('a -> state -> state -> 'a) -> 'a -> t -> 'a
end

module MakeND (S : OrderedType) (I : OrderedType) = struct

  type state = S.t
  type input = I.t
  
  module Smap = Map.Make(S)

  module Transition = struct
    type t = 
      | Epsilon
      | Input of I.t
    let compare x y = match (x,y) with
      | Epsilon, Epsilon -> 0
      | Epsilon, _ -> -1
      | _, Epsilon -> 1
      | Input ix, Input iy -> I.compare ix iy
  end

  open Transition
			
  module Tmap = Map.Make(Transition)

  type t = {
    start : S.t;
    trans : S.t Tmap.t Smap.t }

  let empty s = { start = s; trans = Smap.empty }

  let add_transition aut sx i sy =
    let transx = try Smap.find sx aut with Not_found -> Tmap.empty in
    Smap.add sx (Tmap.add (Input i) sy transx) aut

  let add_epsilon_transition aut sx sy =
    let transx = try Smap.find sx aut with Not_found -> Tmap.empty in
    Smap.add sx (Tmap.add Epsilon sy transx) aut

(***
  let fold_states aut f 

  let fold_transitions aut 
***)

end

(* Deterministic automata. *)

module type D = sig
  type t
  type state
  type input

  val empty : state -> t
  val add_transition : t -> state -> input -> state -> t

  val fold_states : ('a -> state -> 'a) -> 'a -> t -> 'a
  val fold_transitions : 
    ('a -> state -> input -> state -> 'a) -> 'a -> t -> 'a
end

module MakeD (S : OrderedType) (I : OrderedType) = struct

  type state = S.t
  type input = I.t
  
  module Smap = Map.Make(S)
  module Imap = Map.Make(I)

  type t = {
    start : S.t;
    trans : S.t Imap.t Smap.t }

  let empty s = { start = s; trans = Smap.empty }

end

