
(*s Automata. *)

module type OrderedType = sig
  type t
  val compare : t -> t -> int
end

(*s Non-deterministic automata. *)

module type ND = sig
  type t
  type state
  type input

  val empty : state -> t
  val add_transition : t -> state -> input -> state -> t
  val add_epsilon_transition : t -> state -> state -> t

  val fold_states : ('a -> state -> 'a) -> 'a -> t -> 'a
  val fold_transitions : 
    ('a -> state -> input -> state -> 'a) ->
      ('a -> state -> state -> 'a) -> 'a -> t -> 'a
end

module MakeND : 
  functor (S : OrderedType) ->
    functor (I : OrderedType) -> 
      ND with type state = S.t and type input = I.t

(* Deterministic automata. *)

module type D = sig
  type t
  type state
  type input

  val empty : state -> t
  val add_transition : t -> state -> input -> state -> t

  val fold_states : ('a -> state -> 'a) -> 'a -> t -> 'a
  val fold_transitions : 
    ('a -> state -> input -> state -> 'a) -> 'a -> t -> 'a
end

module MakeD : 
  functor (S : OrderedType) ->
    functor (I : OrderedType) -> 
      D with type state = S.t and type input = I.t
