
(*s Datatype of grammars. *)

type terminal = string

type non_terminal = string

type letter = 
  | Terminal of terminal
  | NonTerminal of non_terminal

type production = letter list

type rule = non_terminal * production

module Ntset : Set.S with type elt = non_terminal
module Ntmap : Map.S with type key = non_terminal

module Tset : Set.S with type elt = terminal
module Tmap : Map.S with type key = terminal

module Rset : Set.S with type elt = rule

(*s Null, first and next. *)

type nulls = Ntset.t

type firsts = Tset.t Ntmap.t

type nexts = Tset.t Ntmap.t

(*s Grammar with pre-computed terminals, non-terminals, nulls, firsts and
    nexts *)

type t = {
  start : non_terminal;
  rules : rule list;
  terminals : Tset.t;
  non_terminals : Ntset.t;
  nulls : nulls;
  firsts : firsts;
  nexts : nexts
}

val create : non_terminal -> rule list -> t

val first_production : t -> production -> Tset.t

val is_null_production : t -> production -> bool

(*s Pretty-print. *)

open Format

val pp : formatter -> t -> unit

val pp_production : formatter -> production -> unit
val pp_rule : formatter -> rule -> unit

val pp_ntset : formatter -> Ntset.t -> unit
val pp_tset  : formatter -> Tset.t -> unit

val pp_null : formatter -> nulls -> unit
val pp_first : formatter -> firsts -> unit
val pp_next : formatter -> nexts -> unit

