
{
  open Printf
  open Lexing
  open Gram_parser

  let tokens = Hashtbl.create 97
  let add_token s = Hashtbl.add tokens s ()
  let is_token s = Hashtbl.mem tokens s
}

let space = [' ' '\t' '\r' '\n']
let ident = 
  ['A'-'Z' 'a'-'z' '\192'-'\214' '\216'-'\246' '\248'-'\255']
  ['A'-'Z' 'a'-'z' '_' '\192'-'\214' '\216'-'\246' '\248'-'\255' '\'' '0'-'9']*

rule token = parse
  | space+ { token lexbuf }
  | ident  { let s = lexeme lexbuf in if is_token s then STRING s else IDENT s }
  | '"' ([^ '"' '#']* as s) '"'
           { STRING s }
  | '|'    { BAR }
  | ":" | "::="  
           { PROD }
  | ';'    { SEMI }
  | '#' [^ '\n']* '\n' { token lexbuf }

  (* hacks to parse yacc grammars *)
  | "%{"     { skip_preamble lexbuf; token lexbuf }
  | "{"      { skip_action lexbuf; token lexbuf }
  | "%token" (space+ "<" [^ '>']* ">")?
             { read_tokens lexbuf; token lexbuf }
  | "%prec" space+ ident
             { token lexbuf }

  | eof    { EOF }
  | _      { eprintf "Lexical error character %d\n" (lexbuf.lex_abs_pos);
	     exit 1 }

and skip_preamble = parse
  | "%}" { () }
  | eof  { () }
  | _    { skip_preamble lexbuf }

and skip_action = parse
  | "}"  { () }
  | eof  { () }
  | "{"  { skip_action lexbuf; skip_action lexbuf }
  | _    { skip_action lexbuf }

and read_tokens = parse
  | ident  { add_token (lexeme lexbuf); read_tokens lexbuf }
  | '\n'   { () }
  | _      { read_tokens lexbuf }

{
  let parse_grammar c = grammar token (from_channel c)
}
