
open Format
open Util
open Gram

(*s Expansion table. *)

type expansion_table = Rset.t Tmap.t Ntmap.t

module PPExpansionTable = 
  Table.Make(struct type t = non_terminal let pp s = s end)
            (struct type t = terminal let pp s = s end)
	    (struct type t = Rset.t let pp _ = "E" end)
	    (Ntmap)
	    (Tmap)

(* let pp_table = PPExpansionTable.pp_table *)

let pp_table fmt t =
  Ntmap.iter
    (fun nt m ->
       fprintf fmt "@[Expansions for %s:@\n" nt;
       Tmap.iter 
	 (fun t rs ->
	    fprintf fmt "@[  %s: @[" t;
	    pp_list (fun fmt (_,p) -> pp_production fmt p) comma 
	      fmt (Rset.elements rs);
	    fprintf fmt "@]@]@\n")
	 m;
       fprintf fmt "@]")
    t

let expansions g =
  let add_entry table nt t rule =
    let line = try Ntmap.find nt table with Not_found -> Tmap.empty in
    let s = try Tmap.find t line with Not_found -> Rset.empty in
    Ntmap.add nt (Tmap.add t (Rset.add rule s) line) table
  in
  List.fold_left 
    (fun table ((nt,p) as rule) -> 
       let ts = first_production g p in
       let ts' = 
	 if is_null_production g p then 
	   Tset.union ts (Ntmap.find nt g.nexts)
	 else 
	   ts 
       in
       Tset.fold (fun t table -> add_entry table nt t rule) ts' table)
    Ntmap.empty g.rules

let is_ll1 t =
  try
    Ntmap.iter 
      (fun _ m -> 
	 Tmap.iter (fun _ rs -> if Rset.cardinal rs > 1 then raise Exit) m) 
      t;
    true
  with Exit -> 
    false

(*s LL(1) analyzer. *)

let analyze g = 
  printf "LL(1) analyzer:@\n===============@\n";
  let t = expansions g in
  printf "@[%a@]" pp_table t;
  printf "@\n";
  printf "The grammar is %sLL(1)@\n" (if is_ll1 t then "" else "not ");
  flush stdout






