
open Format
open Arg
open Gram_lexer
open Gram

(*s Main program. Reads grammars from files or standard input and analyses
    them. *)

let files = ref []

let ll1 = ref false
let lr0 = ref false
let slr1 = ref false

let lr () = lr0 := true; slr1 := true
let all () = ll1 := true; lr ()

let speclist = [ "-ll", Set ll1, "perform LL(1) analysis";
		 "-ll1", Set ll1, "perform LL(1) analysis";
		 "-lr0", Set lr0, "perform LR(0) analysis";
		 "-slr1", Set slr1, "perform SLR(1) analysis";
	         "-lr", Unit lr, "perform all LR analyses";
		 "-all", Unit all, "perform all analyses"
	       ]

let analyze g = 
  Gram.pp std_formatter g; flush stdout;
  pp_null std_formatter g.nulls;
  printf "@\n";
  pp_first std_formatter g.firsts;
  printf "@\n";
  pp_next std_formatter g.nexts;
  printf "@\n";
  if !ll1 then begin Ll1.analyze g; printf "@\n"; end;
  if !lr0 then begin Lr.analyze_lr0 g; printf "@\n"; end;
  if !slr1 then Lr.analyze_slr1 g

let analyze_stdin () =
  let s,r = parse_grammar stdin in
  let g = Gram.create s r in
  analyze g

let analyze_file f =
  let c = open_in f in
  try
    let s,r = parse_grammar c in
    close_in c;
    let g = Gram.create s r in
    analyze g
  with e ->
    close_in c;
    raise e

let main () = 
  parse speclist (fun s -> files := s :: !files) "usage: gram [files]";
  match !files with
    | [] -> analyze_stdin ()
    | l  -> List.iter analyze_file (List.rev l)

let _ = Printexc.catch main ()

